package com.sec.xinhua.gateway.beans;

/**
 * 网关的响应状态编码
 */
public enum ApiCode {
    /**
     * 系统部分
     */
    //成功
    SUCCESS(200, "OK"),
    //网关内部错误
    GATEWAY_ERROR(999000, "GATEWAY_ERROR"),
    //签名验证错误
    SIGN_ERROR(999001, "SIGN_ERROR"),

    /**
     * API部分
     */
    //无效的API请求，基本是不符合格式的请求
    API_INVALID_REQUEST(999301, "API_INVALID_REQUEST"),

    //没有找到API配置信息
    API_NOT_FOUND(999302, "API_NOT_FOUND"),
    //API未授权
    API_NON_AUTHORIZED(999303, "API_NON_AUTHORIZED"),
    //API流量限制
    //API_FLOW_LIMITED替换为：操作太频繁啦，请稍后再试
    API_FLOW_LIMITED(999304, "操作太频繁啦，请稍后再试"),

    /**
     * 服务部分
     */
    //服务调用超时
    SERVICE_TIME_OUT(999401, "SERVICE_TIME_OUT"),
    //服务调用出现错误，例如网络断开
    SERVICE_INVOKE_ERROR(999402, "SERVICE_INVOKE_ERROR"),
    //没有找到服务
    SERVICE_NOT_FOUND(999403, "SERVICE_NOT_FOUND"),
    //服务返回格式不对
    SERVICE_RESPONSE_FORMAT_ERROR(999404, "SERVICE_RESPONSE_FORMAT_ERROR");
    private int    code;

    private String msg;

    private ApiCode(int code, String msg) {
        this.code = code;
        this.msg = msg;
    }

    public int getCode() {
        return code;
    }

    public String getMsg() {
        return msg;
    }

    public static enum ApiSubCode {

        //黑名单限制
        SUB_FLOW_LIMITED_BLACKIP(9993040, "api flow limited by black ip"),
        //单机并发量限制
        SUB_FLOW_LIMITED_CONCURRENT_COUNT(9993041, "api flow limited by concurrent count"),
        //全局ip限制
        SUB_FLOW_LIMITED_GLOBAL_IP_COUNT(9993042, "api flow limited by gloal ip count"),
        //api ip限制
        SUB_FLOW_LIMITED_API_IP_COUNT(9993043, "api flow limited by api ip count"),
        //全局uid限制
        SUB_FLOW_LIMITED_GLOBAL_UID_COUNT(9993044, "api flow limited by gloal uid count"),
        //api uid限制
        SUB_FLOW_LIMITED_API_UID_COUNT(9993045, "api flow limited by api uid count"),

        SUB_INVALID_PARAM(9993010, "invalid request invalid param"),

        SUB_INVALID_SIGN(9993011, "invalid request sign not equal"),

        SUB_INVALID_TIMESTAMP(9993012, "invalid request time diff"),

        SUB_INVALID_GET_REQUEST(9993013, "invalid request get method"),

        SUB_INVALID_BODY(9993014, "invalid request body"),

        //无效的apk，不能找到apkmd5
        SUB_APP_PACKAGE_NOT_FOUND(9993015, "APP PACKAGE NOT FOUND"),

        SUB_APP_PACKAGE_SID_NOT_EQUAL(9993016, "APP PACKAGE INVALID SID"),

        SUB_CLIENT_SOURCE_INVALID(9993017, "INVALID CLIENT SOURCE"),

        SUB_INVALID_EXP(9993018, "invalid request by expression");

        private int    code;

        private String msg;

        private ApiSubCode(int code, String msg) {
            this.code = code;
            this.msg = msg;
        }

        public int getCode() {
            return code;
        }

        public String getMsg() {
            return msg;
        }
    }
}
