package com.sec.xinhua.gateway.beans;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;

import com.alibaba.fastjson.JSON;
import com.sec.xinhua.gateway.config.GatewaySettingRepository;
import com.sec.xinhua.gateway.utils.URLHelper;

/**
 * API错误提示的缓存
 *
 */
public class ApiResultCache {

    private static Map<Integer, ApiResult> API_RESULT_CACHE       = new HashMap<Integer, ApiResult>();

    private static Map<Integer, String>    API_RESULT_CHARS_CACHE = new HashMap<Integer, String>();

    static {
        {
            ApiResult result = new ApiResult(ApiCode.GATEWAY_ERROR.getCode(), ApiCode.GATEWAY_ERROR.getMsg());
            API_RESULT_CACHE.put(ApiCode.GATEWAY_ERROR.getCode(), result);
            API_RESULT_CHARS_CACHE.put(ApiCode.GATEWAY_ERROR.getCode(), JSON.toJSONString(result));
        }

        {
            ApiResult result = new ApiResult(ApiCode.API_INVALID_REQUEST.getCode(),
                ApiCode.API_INVALID_REQUEST.getMsg());
            API_RESULT_CACHE.put(ApiCode.API_INVALID_REQUEST.getCode(), result);
            API_RESULT_CHARS_CACHE.put(ApiCode.API_INVALID_REQUEST.getCode(), JSON.toJSONString(result));
        }

        {
            ApiResult result = new ApiResult(ApiCode.API_NOT_FOUND.getCode(), ApiCode.API_NOT_FOUND.getMsg());
            API_RESULT_CACHE.put(ApiCode.API_NOT_FOUND.getCode(), result);
            API_RESULT_CHARS_CACHE.put(ApiCode.API_NOT_FOUND.getCode(), JSON.toJSONString(result));
        }

        {
            ApiResult result = new ApiResult(ApiCode.API_NON_AUTHORIZED.getCode(), ApiCode.API_NON_AUTHORIZED.getMsg());
            API_RESULT_CACHE.put(ApiCode.API_NON_AUTHORIZED.getCode(), result);
            API_RESULT_CHARS_CACHE.put(ApiCode.API_NON_AUTHORIZED.getCode(), JSON.toJSONString(result));
        }

        {
            ApiResult result = new ApiResult(ApiCode.API_FLOW_LIMITED.getCode(), ApiCode.API_FLOW_LIMITED.getMsg());
            API_RESULT_CACHE.put(ApiCode.API_FLOW_LIMITED.getCode(), result);
            API_RESULT_CHARS_CACHE.put(ApiCode.API_FLOW_LIMITED.getCode(), JSON.toJSONString(result));
        }

        {
            ApiResult result = new ApiResult(ApiCode.SERVICE_TIME_OUT.getCode(), ApiCode.SERVICE_TIME_OUT.getMsg());
            API_RESULT_CACHE.put(ApiCode.SERVICE_TIME_OUT.getCode(), result);
            API_RESULT_CHARS_CACHE.put(ApiCode.SERVICE_TIME_OUT.getCode(), JSON.toJSONString(result));
        }

        {
            ApiResult result = new ApiResult(ApiCode.SERVICE_INVOKE_ERROR.getCode(),
                ApiCode.SERVICE_INVOKE_ERROR.getMsg());
            API_RESULT_CACHE.put(ApiCode.SERVICE_INVOKE_ERROR.getCode(), result);
            API_RESULT_CHARS_CACHE.put(ApiCode.SERVICE_INVOKE_ERROR.getCode(), JSON.toJSONString(result));
        }

        {
            ApiResult result = new ApiResult(ApiCode.SERVICE_NOT_FOUND.getCode(), ApiCode.SERVICE_NOT_FOUND.getMsg());
            API_RESULT_CACHE.put(ApiCode.SERVICE_NOT_FOUND.getCode(), result);
            API_RESULT_CHARS_CACHE.put(ApiCode.SERVICE_NOT_FOUND.getCode(), JSON.toJSONString(result));
        }

        {
            ApiResult result = new ApiResult(ApiCode.SERVICE_RESPONSE_FORMAT_ERROR.getCode(),
                ApiCode.SERVICE_RESPONSE_FORMAT_ERROR.getMsg());
            API_RESULT_CACHE.put(ApiCode.SERVICE_RESPONSE_FORMAT_ERROR.getCode(), result);
            API_RESULT_CHARS_CACHE.put(ApiCode.SERVICE_RESPONSE_FORMAT_ERROR.getCode(), JSON.toJSONString(result));
        }

        {
            ApiResult result = new ApiResult(ApiCode.SIGN_ERROR.getCode(), ApiCode.SIGN_ERROR.getMsg());
            API_RESULT_CACHE.put(ApiCode.SIGN_ERROR.getCode(), result);
            API_RESULT_CHARS_CACHE.put(ApiCode.SIGN_ERROR.getCode(), JSON.toJSONString(result));
        }

    }

    public static ApiResult getApiResult(ApiCode apiCode) {
        ApiResult result = API_RESULT_CACHE.get(apiCode.getCode());
        if (result == null) {
            result = new ApiResult(apiCode.getCode(), apiCode.getMsg(), new HashMap<String, String>());
        }
        return result;
    }

    public static ApiResult getApiResult(ApiCode apiCode, String detail) {
        //线上模式不输出详细信息
        if (GatewaySettingRepository.getInstance().getObject().getRunMode().equalsIgnoreCase("online")) {
            return getApiResult(apiCode);
        }

        //详细信息为空不输出
        if (StringUtils.isEmpty(detail)) {
            return getApiResult(apiCode);
        }
        ApiResult result = new ApiResult(apiCode.getCode(), apiCode.getMsg() + "; " + detail);
        return result;
    }

    public static ApiResult getApiResult(ApiCode apiCode, String appendMsg, Throwable e) {
        //线上模式不输出详细信息
        if (GatewaySettingRepository.getInstance().getObject().getRunMode().equalsIgnoreCase("online")) {
            return getApiResult(apiCode);
        }

        //详细信息为空不输出
        if (e == null) {
            return getApiResult(apiCode);
        }

        ApiResult result = new ApiResult(apiCode.getCode(), apiCode.getMsg() + "; " + appendMsg + ";"
                                                            + URLHelper.getStackTrace(e));
        return result;
    }

    public static String getApiResultChars(ApiCode apiCode) {
        return API_RESULT_CHARS_CACHE.get(apiCode.getCode());
    }
}
