package com.sec.xinhua.gateway.config.local;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.net.URLConnection;

import com.alibaba.fastjson.JSON;
import com.sec.xinhua.gateway.config.ConfigLoader;
import com.sec.xinhua.gateway.utils.SysConstants;

/**
 * 基于文件的配置加载器
 *
 */
public class FileConfigLoader implements ConfigLoader {

    protected String filePath;

    public FileConfigLoader(String filePath) {
        this.filePath = filePath;
    }

    @Override
    public String toString() {
        return "[FileConfigLoader] file:" + filePath;
    }

    @Override
    public String load() {
        File file = new File(filePath);
        if (file.exists()) {
            return readFile(file);
        } else {
            try {
                file.createNewFile();
            } catch (IOException e) {
            }
        }
        return null;
    }

    public boolean flush(Object obj) {
        return writeFile(JSON.toJSONString(obj, true), filePath);
    }

    protected boolean writeFile(final String content, final String fileName) {
        boolean result = true;
        try {

            String tmpFile = fileName + ".tmp";
            writeFileNotSafe(content, tmpFile);

            String bakFile = fileName + ".bak";
            String prevContent = readFile(new File(fileName));
            if (prevContent != null) {
                writeFileNotSafe(prevContent, bakFile);
            }

            File file = new File(fileName);
            file.delete();

            file = new File(tmpFile);
            file.renameTo(new File(fileName));
        } catch (Exception e) {
            result = false;
        }
        return result;
    }

    protected void writeFileNotSafe(final String str, final String fileName) throws IOException {
        File file = new File(fileName);
        File fileParent = file.getParentFile();
        if (fileParent != null) {
            fileParent.mkdirs();
        }
        FileWriter fileWriter = null;

        try {
            fileWriter = new FileWriter(file);
            fileWriter.write(str);
        } catch (IOException e) {
            throw e;
        } finally {
            if (fileWriter != null) {
                try {
                    fileWriter.close();
                } catch (IOException e) {
                    throw e;
                }
            }
        }
    }

    protected String readFile(final URL url) {
        InputStream in = null;
        String result = null;
        try {
            URLConnection urlConnection = url.openConnection();
            urlConnection.setUseCaches(false);
            in = urlConnection.getInputStream();
            result = readFile(in);
        } catch (Exception e) {
        } finally {
            if (in != null) {
                try {
                    in.close();
                } catch (IOException e) {
                }
            }
        }

        return result;
    }

    protected String readFile(final File file) {
        if (file == null) {
            return null;
        }

        String result = null;
        if (file.exists()) {
            InputStream in = null;
            try {
                in = new FileInputStream(file);
                result = readFile(in);
            } catch (Exception e) {
            } finally {
                if (in != null) {
                    try {
                        in.close();
                    } catch (IOException e) {
                    }
                }
            }
        }

        return result;
    }

    protected String readFile(final InputStream in) throws IOException, UnsupportedEncodingException {
        int len = in.available();
        byte[] data = new byte[len];
        in.read(data, 0, len);
        return new String(data, SysConstants.ENCODE);
    }

}
