package com.sec.xinhua.gateway.handler;

import java.util.ArrayList;
import java.util.List;

import com.sec.xinhua.gateway.beans.ApiCode;
import com.sec.xinhua.gateway.beans.ApiContext;
import com.sec.xinhua.gateway.beans.ApiResult;
import com.sec.xinhua.gateway.beans.ApiResultCache;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * 默认的的handler调用链
 *
 */
public class DefaultApiHandlerChain implements ApiHandlerChain {

    private static final Log logger = LogFactory.getLog("errorLogger");

    private static List<ApiHandler> HANDLERS;

    private static DefaultApiHandlerChain instance = new DefaultApiHandlerChain();

    static {
        HANDLERS = new ArrayList<ApiHandler>();
        /**
         * 参数校验
         */
        HANDLERS.add(new SysParamValidateHandler());
        /**
         * dubbo参数转换
         */
        HANDLERS.add(new DubboArgumentConvertHandler());
        /**
         * dubbo服务调用
         */
        HANDLERS.add(new DubboServiceInvokeHandler());
    }

    public static DefaultApiHandlerChain getInstance() {
        return instance;
    }

    @Override
    public ApiResult doHandle(ApiContext apiContext) {
        ApiResult apiResult = null;
        try {
            for (int i = 0; i < HANDLERS.size(); i++) {
                apiResult = HANDLERS.get(i).handle(apiContext);
                if (apiResult != null) {
                    break;
                }
            }
        } catch (Exception e) {
            String error = "[Do handler error]";
            logger.error(error, e);
            apiResult = ApiResultCache.getApiResult(ApiCode.GATEWAY_ERROR, error, null);
        }

        //所有handler都处理完了，结果必须不能为空
        if (apiResult == null) {
            String error = "[Do handler error]All handler execute finished,but apiResult was null";
            logger.error(error);
            apiResult = ApiResultCache.getApiResult(ApiCode.GATEWAY_ERROR, error);
        }
        return apiResult;
    }
}
