package com.sec.xinhua.gateway.handler;

import com.alibaba.dubbo.rpc.RpcContext;
import com.alibaba.dubbo.rpc.service.GenericService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.sec.xinhua.gateway.beans.*;
import com.sec.xinhua.gateway.config.GatewaySettingRepository;
import com.sec.xinhua.gateway.service.DubboServiceManager;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.Map;

/**
 * dubbo 调用
 * by Zhangqx 2016.03.30
 */
public class DubboServiceInvokeHandler implements ApiHandler {
    private static final Log gatewayLogger = LogFactory.getLog("riskGatewayLogger");
    private static final Log errorLogger = LogFactory.getLog("errorLogger");

    @Override
    public ApiResult handle(ApiContext apiContext) {
        SimpleApi api = apiContext.getApi();
        //获取dubbo服务
        GenericService service = DubboServiceManager.getService(api);
        if (service == null) {
            String error = "[Can not found Service];serviceName:" + api.getServiceName() + ",serviceVersion:"
                    + api.getServiceVersion();
            errorLogger.error(error);
            return ApiResultCache.getApiResult(ApiCode.SERVICE_NOT_FOUND, error);
        }

        //调用dubbo服务
        Object invokeResult = null;
        Map<String, String> map = apiContext.getSysParam().getMap();
        if (map == null) {
            errorLogger.error("[Risk params map is null]");
        } else {
            try {
                String riskParams = JSON.toJSONString(map);
                RpcContext.getContext().setAttachment("riskParams", riskParams);
            } catch (Exception e) {
                errorLogger.error("[Risk params convert to json fail]", e);
            }
        }
        try {
            invokeResult = service.$invoke(api.getMethod(), apiContext.getInvokeTypes(), apiContext.getInvokeValues());
        } catch (Throwable e) {
            String error = "[Invoke dubbo service error]";
            errorLogger.error(error, e);
            return ApiResultCache.getApiResult(ApiCode.SERVICE_INVOKE_ERROR, error, null);
        } finally {
            RpcContext.getContext().clearAttachments();
        }


        //请求和响应日志输出
        try {
            if (GatewaySettingRepository.getInstance().getObject().isLogInvokeResult()) {
                StringBuilder sb = new StringBuilder("[Invoke detail]");
                sb.append("serviceName:").append(api.getServiceName()).append(",");
                sb.append("serviceVersion:").append(api.getServiceVersion()).append(",");
                sb.append("method:").append(api.getMethod()).append(",");
                sb.append("invokeTypes:").append(JSON.toJSONString(apiContext.getInvokeTypes(), false)).append(",");
                sb.append("invokeResult:").append(invokeResult == null ? null : JSON.toJSONString(invokeResult, false));
                gatewayLogger.info(sb.toString());
            }
        } catch (Exception e) {
            errorLogger.error("Log invoke detail error", e);
        }
        ApiResult apiResult = ApiResultCache.getApiResult(ApiCode.SUCCESS, null);
        if (invokeResult != null) {
            JSONObject jsonObj = null;
            try {
                jsonObj = (JSONObject) JSON.toJSON(invokeResult);
                if (jsonObj.get("class") != null) {
                    jsonObj.remove("class");
                }
                if (jsonObj.get("data") != null) {
                    JSONObject dataObj = (JSONObject) jsonObj.get("data");
                    if (dataObj != null && dataObj.get("class") != null) {
                        dataObj.remove("class");
                        jsonObj.put("data", dataObj);
                    }
                }
                apiResult.setOriginResult(jsonObj);
                apiResult.setData(jsonObj);
            } catch (Exception e) {
                apiResult.setOriginResult(jsonObj);
                apiResult.setData(invokeResult);
            }
        }
        return apiResult;
    }

}
