package com.sec.xinhua.gateway.handler;

import com.alibaba.fastjson.JSON;
import com.sec.xinhua.gateway.beans.*;
import com.sec.xinhua.gateway.config.GatewaySettingRepository;
import com.sec.xinhua.gateway.parse.HttpParamConstants;
import com.sec.xinhua.gateway.parse.ParamParser;
import com.sec.xinhua.gateway.parse.RequestParam;
import com.sec.xinhua.gateway.utils.ParamParseException;
import com.sec.xinhua.gateway.utils.URLHelper;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * @author zhangqx
 * @version $Id: RequestHandler.java, v 0.1 2015年12月9日 上午11:02:26 zhangqx Exp $
 */
public class RequestHandler {
    private static final Log statsLogger = LogFactory.getLog("riskStatsLogger");
    private static final Log logger = LogFactory.getLog("riskGatewayLogger");
    private static final Log errorLogger = LogFactory.getLog("errorLogger");

    /*
     * http参数的解析,service调用
     */
    public void httpRequestHandle(HttpServletRequest req, HttpServletResponse response) {
        ApiContext apiContext = new ApiContext();
        long startTime = System.currentTimeMillis();
        RequestParam requestParam = null;
        try {
            requestParam = ParamParser.parseHttpParams(req);
            apiContext.setSysParam(requestParam.getSysParam());
            apiContext.setHttpParam(requestParam);
        } catch (ParamParseException e) {
            StringBuilder sb = new StringBuilder("[Request Parse Error]");
            sb.append(" requestTime:").append(URLHelper.getCurrentTime()).append(",");
            sb.append(" clientIp:").append(URLHelper.getIpAddr(req)).append(",");
            sb.append(" url:").append(req.getRequestURI().toString() + "?" + req.getQueryString()).append(",");
            sb.append(" detail:").append(e.getMessage());
            errorLogger.info(sb.toString(), e);
            ApiResult apiResult = ApiResultCache.getApiResult(ApiCode.API_INVALID_REQUEST,
                    "Parse error:" + e.getMessage(), null);
            print(response, JSON.toJSONString(apiResult));
            this.logRequestStatistics(apiContext, apiResult, startTime);
        }

        // 原始返回
//        GatewaySetting gatewaySetting = GatewaySettingRepository.getInstance().getObject();
        ApiResult invokeResult = serviceInvoke(apiContext);
//        if (gatewaySetting != null) {
//            String originResult = gatewaySetting.getOriginResult();
//            if (StringUtils.isNotEmpty(originResult)) {
//                String curApi = req.getParameter(HttpParamConstants.API);
//                String curApi = "xinhua.sec.data";
        // 有些api需要直接返回dubbo返回的值,不需要包装apiCode和apiMsg;
//                if (originResult.contains(curApi)) {
        print(response, invokeResult.getOriginResult() == null ? null : invokeResult.getOriginResult().toString());
        logRequestStatistics(apiContext, invokeResult, startTime);
        return;
//                }
//            }
//        }
//        invokeResult.setOriginResult(null);
//        print(response, JSON.toJSONString(invokeResult));
//        logRequestStatistics(apiContext, invokeResult, startTime);
    }

    /*
     * 用于记录请求耗时
     */
    private void logRequestStatistics(ApiContext apiContext, ApiResult apiResult, long startTime) {
        if (apiContext == null || apiContext.getSysParam() == null || apiResult == null) {
            return;
        }
        try {
            if (StringUtils.isNotBlank(apiContext.getSysParam().getApi())
                    && StringUtils.isNotBlank(apiContext.getSysParam().getApiVersion())) {
                StringBuilder sb = new StringBuilder();
                sb.append(apiContext.getSysParam().getApi()).append("_")
                        .append(apiContext.getSysParam().getApiVersion()).append(",");
                sb.append(apiResult.getApiCode()).append(",");
                sb.append(System.currentTimeMillis() - startTime);
                statsLogger.info(sb.toString());
            }
        } catch (Exception e1) {
            errorLogger.error("Log stats error", e1);
        }
    }

    /*
     * 响应流
     */
    private void print(HttpServletResponse response, String content) {
        response.setHeader("Access-Control-Allow-Origin", "*");
        response.setHeader("Access-Control-Allow-Methods", "GET,PUT,DELETE,POST,OPTIONS");
        response.setHeader("Access-Control-Allow-Headers", "Origin,X-Requested-With,Content-Type,Accept,kopds");
        try {
            response.getWriter().print(content);
            response.getWriter().flush();
        } catch (IOException e) {
            errorLogger.error("Print response error,response content:" + content, e);
        }
    }

    /*
     * 调用handler链,泛化调用dubbo服务
     */
    private ApiResult serviceInvoke(ApiContext apiContext) {
        ApiResult apiResult = null;
        try {
            apiResult = DefaultApiHandlerChain.getInstance().doHandle(apiContext);
        } catch (Exception e) {
            errorLogger.error(e.getMessage(), e);
            apiResult = ApiResultCache.getApiResult(ApiCode.API_INVALID_REQUEST, "System error:" + e.getMessage(), null);
        } finally {
            //记录响应结果
            try {
                if (GatewaySettingRepository.getInstance().getObject().isLogResponse()) {
                    StringBuilder sb = new StringBuilder("[Response detail]");
                    if (apiResult.getApiCode() == ApiCode.SUCCESS.getCode()) {
                        sb.append("Y");
                    } else {
                        sb.append("N");
                    }
                    if (apiContext.getApi() != null) {
                        sb.append(", api:").append(apiContext.getApi().getApiName()).append("_")
                                .append(apiContext.getApi().getApiVersion());
                    }
                    sb.append(", response:").append(JSON.toJSONString(apiResult));
                    logger.info(sb.toString());
                }
            } catch (Exception e) {
                errorLogger.error("Log response error", e);
            }
        }
        return apiResult;
    }

    public ApiResult thriftRequestHandle(String apiName, String apiVersion, String jsonData) {
        ApiContext apiContext = new ApiContext();
        long startTime = System.currentTimeMillis();
        RequestParam requestParam = null;
        try {
            requestParam = ParamParser.parseThriftParams(apiName, apiVersion, jsonData);
            apiContext.setSysParam(requestParam.getSysParam());
            apiContext.setHttpParam(requestParam);
        } catch (ParamParseException e) {
            StringBuilder sb = new StringBuilder("[Request Parse Error]");
            sb.append(" requestTime:").append(URLHelper.getCurrentTime()).append(",");
            sb.append(" detail:").append(e.getMessage());
            errorLogger.info(sb.toString(), e);
            ApiResult apiResult = ApiResultCache.getApiResult(ApiCode.API_INVALID_REQUEST,
                    "Parse error:" + e.getMessage(), null);
            this.logRequestStatistics(apiContext, apiResult, startTime);
        }
        ApiResult apiResult = serviceInvoke(apiContext);
        logRequestStatistics(apiContext, apiResult, startTime);
        return apiResult;
    }
}
