package com.sec.xinhua.gateway.handler;

import com.sec.xinhua.gateway.beans.*;
import com.sec.xinhua.gateway.config.ConfigRepositoryFactory;
import com.sec.xinhua.gateway.config.remote.ApiRemoteRepository;
import com.sec.xinhua.gateway.parse.HttpParamConstants;
import com.sec.xinhua.gateway.parse.SysParam;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * 平台参数校验
 */
public class SysParamValidateHandler implements ApiHandler {

    private static final Log logger  = LogFactory.getLog("errorLogger");

    private ApiRemoteRepository apiRepo = ConfigRepositoryFactory.getApiRepository();

    @Override
    public ApiResult handle(ApiContext context) {
        SysParam sp = context.getHttpParam().getSysParam();

        //平台参数校验
        if (StringUtils.isEmpty(sp.getApi()) || StringUtils.isEmpty(sp.getApiVersion())) {
            StringBuilder sb = new StringBuilder("[Invalid sys param]some one can't found,sys param:[");
            sb.append(HttpParamConstants.API).append(":").append(sp.getApi()).append(",");
            sb.append(HttpParamConstants.API_VERSION).append(":").append(sp.getApiVersion()).append(",");
            sb.append("]");
            String error = sb.toString();
            logger.info(error);
            return ApiResultCache.getApiResult(ApiCode.API_INVALID_REQUEST, error);
        }

        //检查API是否存在
        SimpleApi api = apiRepo.getObject(sp.getApi() + "_" + sp.getApiVersion());
        if (api == null) {
            logger.info("[Can not found api]api:" + api + ",apiVersion:" + sp.getApiVersion());
            return ApiResultCache.getApiResult(ApiCode.API_NOT_FOUND, null);
        }
        context.setApi(api);
        return null;
    }
}
