package com.sec.xinhua.gateway.parse;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.sec.xinhua.gateway.beans.SimpleApi;
import com.sec.xinhua.gateway.beans.SimpleApiParam;
import com.sec.xinhua.gateway.config.ConfigRepositoryFactory;
import com.sec.xinhua.gateway.utils.InputStreamCacher;
import com.sec.xinhua.gateway.utils.ParamParseException;
import com.sec.xinhua.gateway.utils.SysConstants;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 参数解析
 *
 * @author zhangqx
 * @version $Id: ParamParser.java, v 0.1 2015年12月9日 上午11:45:58 zhangqx Exp $
 */
public class ParamParser {

    private static final Log parseLog = LogFactory.getLog("parseLogger");
    private static final String API = "xinhua.sec.data";
    private static final String VERSION = "1.0.0";

    /**
     * http参数转化
     */
    public static RequestParam parseHttpParams(HttpServletRequest req) throws ParamParseException {
        SysParam sysParam = new SysParam();
        RequestParam requestParam = new RequestParam();
        String contentType = req.getContentType();
        if (contentType != null && contentType.contains("multipart/form-data")) {
            fileParamsConvert(req, requestParam);
        } else {
            InputStreamCacher streamCacher = null;
            try {
                streamCacher = new InputStreamCacher(req.getInputStream());
                jsonParamsConvert(streamCacher.getInputStream(), requestParam);
            } catch (Exception e1) {
                parseLog.info("Params json parse exception : postBody is null, " + e1);
                try {
                    paramsConvert(streamCacher.convertString(), requestParam);
                } catch (Exception e2) {
                    parseLog.info("Params urlencode parse exception : stream cacher convert string exception, " + e2);
                }
            }
        }
        if (req.getParameter(HttpParamConstants.API) != null) {
            sysParam.setApi(req.getParameter(HttpParamConstants.API));
        } else {
            sysParam.setApi(API);
        }
        sysParam.setApiVersion(VERSION);
        sysParam.setSessionId(req.getSession().getId());
        sysParam.setRefererUrl(req.getHeader("referer"));
        sysParam.setRequestUrl(req.getRequestURI().toString() + "?" + req.getQueryString());
        requestParam.setSysParam(sysParam);
        return requestParam;
    }

    private static void jsonParamsConvert(InputStream input, RequestParam requestParam) throws Exception {
        String postBody = IOUtils.toString(input, SysConstants.ENCODE);
        if (StringUtils.isEmpty(postBody)) {
            throw new ParamParseException("Can not get post param from inputstream , post body is null");
        }
        JSONObject jsonData = JSON.parseObject(postBody);
        if (jsonData == null) {
            throw new ParamParseException("Http Post body parse complete but json was null;post body:<"
                    + postBody + ">");
        }
        for (String key : jsonData.keySet()) {
            String jsonValue = jsonData.getString(key);
            if (jsonValue == null) {
                throw new ParamParseException("Http Param value was null,name:" + key);
            }
            requestParam.putBizParam(key, jsonValue);
        }

    }


    private static void paramsConvert(String paramStr, RequestParam requestParam) {
        if (StringUtils.isEmpty(paramStr)) {
            return;
        }
        for (String pair : paramStr.split("&")) {
            String kv[] = pair.split("=");
            if (kv.length == 2) {
                requestParam.putBizParam(kv[0], kv[1]);
            }
        }
    }

    public static RequestParam parseThriftParams(String apiName, String apiVersion, String jsonData)
            throws ParamParseException {
        SysParam sysParam = new SysParam();
        RequestParam requestParam = new RequestParam();
        sysParam.setApi(apiName);
        sysParam.setApiVersion(apiVersion);
        requestParam.setSysParam(sysParam);
        if (!StringUtils.isEmpty(jsonData)) {
            JSONObject jsonParams = JSON.parseObject(jsonData);
            if (jsonParams == null) {
                throw new ParamParseException("Thrift params parse complete but json was null;post body:<" + jsonData
                        + ">");
            }
            for (String key : jsonParams.keySet()) {
                String jsonValue = jsonParams.getString(key);
                if (jsonValue == null) {
                    throw new ParamParseException("Thrift Param value was null,name:" + key);
                }
                requestParam.putBizParam(key, jsonValue);
            }
        }
        return requestParam;
    }

    private static RequestParam fileParamsConvert(HttpServletRequest request, RequestParam requestParam) {
        Map<String, SimpleApi> apiMap = ConfigRepositoryFactory.getAllApi();
        String apiKey = request.getParameter(HttpParamConstants.API) + "_" + request.getParameter(HttpParamConstants.API_VERSION);
        SimpleApi api = apiMap.get(apiKey);
        List<String> keys = new ArrayList<String>();
        if (api != null) {
            List<SimpleApiParam> lists = api.getParams();
            for (SimpleApiParam param : lists) {
                if (param.getParamType() == 40) {
                    keys.add(param.getSourceName());
                }
            }
        }
        // 超过这个阈值 1M ,文件将被存在临时文件夹中,否则在内存中
        DiskFileItemFactory factory = new DiskFileItemFactory(1024 * 1024 * 2, new File("temp"));
        ServletFileUpload upload = new ServletFileUpload(factory);
        upload.setSizeMax(1024 * 1024 * 2);
        upload.setFileSizeMax(1024 * 512);
        String[] fileExts = {"mp4", "png", "jpg", "avi", "txt", "gif", "jpeg"};
        try {
            List<FileItem> items = upload.parseRequest(request);
            if (items == null) {
                return requestParam;
            }
            int size = items.size();
            int keysSize = keys.size();
            int fileIndex = 0;
            for (int i = 0; i < size; i++) {
                FileItem fileItem = items.get(i);
                String paramName = fileItem.getFieldName();
                if (fileItem.isFormField()) {
                    String paramValue = fileItem.getString("UTF-8");
                    requestParam.putBizParam(paramName, paramValue);
                } else {
                    String fileName = fileItem.getName();
                    String fileExt = fileName.substring(fileName.lastIndexOf(".") + 1, fileName.length());
                    if (Arrays.binarySearch(fileExts, fileExt) != -1 && fileIndex < keysSize) {
                        requestParam.putBizParam(paramName, fileItem.get());
                    } else {
                        parseLog.info("This kind of file type is not allowed , fileName is " + fileName);
                    }
                }
            }
        } catch (Exception e) {
            parseLog.info("Multipart data parse exception ", e);
        }
        return requestParam;
    }
}