package com.sec.xinhua.gateway.service;

import com.alibaba.dubbo.config.ApplicationConfig;
import com.alibaba.dubbo.config.ReferenceConfig;
import com.alibaba.dubbo.config.RegistryConfig;
import com.alibaba.dubbo.rpc.service.GenericService;
import com.sec.xinhua.gateway.beans.GatewaySetting;
import com.sec.xinhua.gateway.beans.SimpleApi;
import com.sec.xinhua.gateway.config.ConfigRepositoryFactory;
import com.sec.xinhua.gateway.config.GatewaySettingRepository;
import com.sec.xinhua.gateway.utils.SysConstants;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

public class DubboServiceManager {
    private static final Log logger = LogFactory.getLog(DubboServiceManager.class);

    private static ApplicationConfig APPLICATION;

    private static RegistryConfig REGISTRY;

    private static ConcurrentHashMap<String, GenericService> SERVICE_CACHE = new ConcurrentHashMap<String, GenericService>();

    static {
        init();
    }

    public static void init() {
        APPLICATION = new ApplicationConfig();
        APPLICATION.setName(SysConstants.APPLICATION_NAME);
        REGISTRY = new RegistryConfig();
        GatewaySetting gs = GatewaySettingRepository.getInstance().getObject();
        REGISTRY.setProtocol("zookeeper");
        REGISTRY.setAddress(gs.getZkAddress());
        REGISTRY.setFile(System.getProperty("user.home") + "/risk_gateway_registery.config");
    }

    public static GenericService getService(SimpleApi simpleApi) {
        String serviceName = simpleApi.getServiceName();
        String serviceVersion = simpleApi.getServiceVersion();
        Integer timeout = simpleApi.getTimeout();
        String cacheKey = serviceName + "_" + serviceVersion + "_" + timeout;
        GenericService service = SERVICE_CACHE.get(cacheKey);
        if (service == null) {
            //使用ConcurrentHashMap避免锁竞争
            try {
                ReferenceConfig<GenericService> reference = new ReferenceConfig<GenericService>();
                reference.setInterface(serviceName);
                reference.setVersion(serviceVersion);
                reference.setGeneric(true);
                reference.setApplication(APPLICATION);
                /**
                 * 因为Gatewaysetting 现在是动态配置,所以这里需要重新获取zk地址
                 */
                GatewaySetting gs = GatewaySettingRepository.getInstance().getObject();
                REGISTRY.setAddress(gs.getZkAddress());
                reference.setRegistry(REGISTRY);
//                reference.setFilter("DubboConsumerStaticFilter");
                reference.setTimeout(timeout);
                reference.setRetries(0);
                service = reference.get();
                SERVICE_CACHE.put(cacheKey, service);
            } catch (Exception e) {
                logger.error(
                        "Get dubbo service error;serviceName:" + serviceName + ",serviceVersion:" + serviceVersion, null);
            }
        }
        return service;
    }

    public static void initAllDubboService() {
        logger.info("start init dubbo services");
        Map<String, SimpleApi> apiMap = ConfigRepositoryFactory.getAllApi();
        Set<String> keys = apiMap.keySet();
        for (String key : keys) {
            long start = System.currentTimeMillis();
            SimpleApi api = apiMap.get(key);
            DubboServiceManager.getService(api);
            long end = System.currentTimeMillis();
            long cost = end - start;
            logger.info("initService " + api.getServiceName() + " " + api.getServiceVersion() + " cost:" + cost);
        }
        logger.info("init dubbo service success");
    }

}
