/**
 * 
 */
package com.sec.xinhua.gateway.utils;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.InitializingBean;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * 
 * @author jerry.xu
 * 
 * @param <T>
 */
public abstract class AbstractLocalCache<T> implements InitializingBean {
    private static final Log        LOCAL_CACHE_LOGGER          = LogFactory.getLog("errorLogger");

    protected List<T>                cache                       = new ArrayList();
    private ScheduledExecutorService loadExcutor                 = new ScheduledThreadPoolExecutor(10);
    //默认60s更新一次
    private long                     latency                     = 60;

    public long getLatency() {
        return latency;
    }

    public void setLatency(long latency) {
        this.latency = latency;
    }

    /**
     * 判断状态是否发生了变更
     *
     * @return
     */
    private boolean changed() {
        //TODO 这里改成true
        return true;
    }

    private class LoadDataTask implements Runnable {

        @Override
        public void run() {
            try {
                if (changed()) {
                    List<T> reload = reLoad();
                    synchronized (cache) {
                        if (reload == null) {
                            cache = new ArrayList();
                        } else {
                            cache = reload;
                        }
                    }
                }
            } catch (Exception e) {
                LOCAL_CACHE_LOGGER.warn("reload cache error",e);
            }
        }
    }

    /**
     * 重新加载到缓存
     */
    protected abstract List<T> reLoad();

    @Override
    public void afterPropertiesSet() throws Exception {
        //延迟1秒加载数据
        loadExcutor.scheduleAtFixedRate(new LoadDataTask(), 1, latency, TimeUnit.SECONDS);
    }

}
